/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _LIGHT_H_
#define _LIGHT_H_

#include <vector>
 
#include "glUtil.h"
#include "vector3.h"
#include "vector4.h"
#include "color.h"

 
class Light
{
public:
	
	Light(int numColor, int status);
	Light(int numColor, float coeff, int status);
	virtual ~Light();

	void setNumColor(int numColor);
	void setNumColor(int numColor, float coeff);

	void setAmbient(Color ambient);
	void setColor(Color color);
		
	void activate(int num);
    virtual void release() = 0;	
			
	int getLightId();
		
	int getStatus();
	void switchOn();
	void switchOff();
	
protected:
	int lightId;
	
	Color mAmbient;
	Color mColor;
		
	/*	0 if the light is off,
		1 if the light is on, 
		2 if the light is alway on */
	int mStatus;
};


class LightPoint : public Light {
public :
	LightPoint(int numColor, int status, Vector3 position);
	LightPoint(int numColor, int status, Vector3 position, float intensity, float d0);
	LightPoint(int numColor, float coeff, int status, Vector3 position, float intensity, float d0);
	~LightPoint();
		
	void activate(int num);
	void release();
		
	void setIntensityAndAttenuation(float intensity, float d0);
	void setAttenuations(float constant, float linear, float quadratic);
		
	void setPosition(Vector3 position);
	
private :
	Vector4 mPosition;
	float constantAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;
};


class LightDir : public Light {
public :	
	LightDir(int numColor, int status, Vector3 direction);
	LightDir(int numColor, float coeff, int status, Vector3 direction);
	~LightDir();
		
	void activate(int num);	
	void release();

	void setDirection(Vector3 direction);
		
private :
	Vector4 mDirection;
};


class LightSpot : public Light {
public :
	LightSpot(int numColor, int status, Vector3 position, Vector3 direction, float exponent, float cutoff);	
	LightSpot(int numColor, int status, Vector3 position, Vector3 direction, float exponent, float cutoff, float intensity, float d0);
	LightSpot(int numColor, float coeff, int status, Vector3 position, Vector3 direction, float exponent, float cutoff, float intensity, float d0);
	~LightSpot();
		
	float getCutoff();
	Vector3 getPosition();
	Vector3 getDirection();
			
	void activate(int num);	
	void release();
		
	void setIntensityAndAttenuation(float intensity, float d0);
	void setAttenuations(float constant, float linear, float quadratic);		

	void setSpotDirection(Vector3 direction);
	void setPosition(Vector3 position);
		
private :	
	Vector4 mPosition;
	Vector3 mDirection;
		
	float constantAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;
		
	float mExponent;
	float mCutoff;
};

typedef std::vector<Light*> LightList;
typedef std::vector<LightPoint*> LightPointList;
typedef std::vector<LightDir*> LightDirList;
typedef std::vector<LightSpot*> LightSpotList;


#endif // _LIGHT_H_
